@@ -7,7 +7,7 @@ use Data::Stream::Bulk;
 use Data::Stream::Bulk::Array;
 use Data::Stream::Bulk::Path::Class;
 use DateTime;
-use Digest::SHA1;
+use Digest::SHA;
 use File::Find::Rule;
 use Git::PurePerl::Actor;
 use Git::PurePerl::Config;
@@ -37,7 +37,7 @@ use IO::Socket::INET;
 use Path::Class;
 use namespace::autoclean;
 
-our $VERSION = '0.47';
+our $VERSION = '0.48';
 $VERSION = eval $VERSION;
 
 has 'directory' => (
@@ -155,6 +155,7 @@ sub ref_names {
     if ( -f $packed_refs ) {
         foreach my $line ( $packed_refs->slurp( chomp => 1 ) ) {
             next if $line =~ /^#/;
+            next if $line =~ /^\^/;
             my ( $sha1, my $name ) = split ' ', $line;
             push @names, $name;
         }
@@ -174,7 +175,6 @@ sub refs {
 
 sub ref_sha1 {
     my ( $self, $wantref ) = @_;
-    my @refs;
     my $dir = dir( $self->gitdir, 'refs' );
     return unless -d $dir;
 
@@ -183,8 +183,7 @@ sub ref_sha1 {
         my $sha1 = file($file)->slurp
             || confess("Error reading $file: $!");
         chomp $sha1;
-        return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
-        return $sha1;
+        return _ensure_sha1_is_sha1( $self, $sha1 );
     }
 
     foreach my $file ( File::Find::Rule->new->file->in($dir) ) {
@@ -193,25 +192,36 @@ sub ref_sha1 {
             my $sha1 = file($file)->slurp
                 || confess("Error reading $file: $!");
             chomp $sha1;
-            return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
-            return $sha1;
+            return _ensure_sha1_is_sha1( $self, $sha1 );
         }
     }
 
     my $packed_refs = file( $self->gitdir, 'packed-refs' );
     if ( -f $packed_refs ) {
+        my $last_name;
+        my $last_sha1;
         foreach my $line ( $packed_refs->slurp( chomp => 1 ) ) {
             next if $line =~ /^#/;
             my ( $sha1, my $name ) = split ' ', $line;
-            if ( $name eq $wantref ) {
-                return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
-                return $sha1;
-            }
+            $sha1 =~ s/^\^//;
+            $name ||= $last_name;
+
+            return _ensure_sha1_is_sha1( $self, $last_sha1 ) if $last_name and $last_name eq $wantref and $name ne $wantref;
+
+            $last_name = $name;
+            $last_sha1 = $sha1;
         }
+        return _ensure_sha1_is_sha1( $self, $last_sha1 ) if $last_name eq $wantref;
     }
     return undef;
 }
 
+sub _ensure_sha1_is_sha1 {
+    my ( $self, $sha1 ) = @_;
+    return $self->ref_sha1($1) if $sha1 =~ /^ref: (.*)/;
+    return $sha1;
+}
+
 sub ref {
     my ( $self, $wantref ) = @_;
     return $self->get_object( $self->ref_sha1($wantref) );
@@ -556,4 +566,3 @@ This module is free software; you can redistribute it or
 modify it under the same terms as Perl itself.
 
 =cut
-
