@@ -2,6 +2,7 @@ package Git::PurePerl::NewObject;
 use Moose;
 use MooseX::StrictConstructor;
 use Moose::Util::TypeConstraints;
+use namespace::autoclean;
 
 enum 'ObjectKind' => qw(commit tree blob tag);
 
@@ -10,11 +11,9 @@ has 'size' => ( is => 'ro', isa => 'Int', required => 0, lazy_build => 1 );
 has 'content' => ( is => 'rw', isa => 'Str', required => 0, lazy_build => 1 );
 has 'sha1'    => ( is => 'ro', isa => 'Str', required => 0, lazy_build => 1 );
 
-__PACKAGE__->meta->make_immutable;
-
 sub _build_sha1 {
     my $self = shift;
-    my $sha1 = Digest::SHA1->new;
+    my $sha1 = Digest::SHA->new;
     $sha1->add( $self->raw );
     my $sha1_hex = $sha1->hexdigest;
     return $sha1_hex;
@@ -30,4 +29,5 @@ sub raw {
     return $self->kind . ' ' . $self->size . "\0" . $self->content;
 }
 
-1;
+__PACKAGE__->meta->make_immutable;
+
